@@ -17,9 +17,12 @@ limitations under the License.
 from __future__ import print_function
 
 import datetime
+import json
 import re
 
 from botocore.exceptions import ClientError
+from netaddr import IPNetwork
+import requests
 
 import ef_utils
 
@@ -573,13 +576,55 @@ class EFAwsResolver(object):
       else:
         return default
 
-  def wafv2_ip_set_id(self, lookup, default=None):
+  def get_cloudflare_cidrs(self):
+    """
+     Args: None
+     Returns:
+       List of Cloudflare CIDR blocks for whitelisting in AWS WAF
+    """
+    r = requests.get('https://cloudflare.com/ips-v4', timeout=10)
+    r.raise_for_status()
+    return r.text.split('\n')
+
+  def waf_cloudflare_ip_list(self):
+    """
+     Args: None
+     Returns:
+       JSON array of WAFv1 IPSetDescriptors containing all Cloudflare CIDR IPs
+     Notes:
+       CIDRs in this array will be broken down to multiple /16's if original prefix is /1-/15
+        due to limitations with WAFv1
+    """
+    cidr_list = []
+    for cidr in self.get_cloudflare_cidrs():
+      net = IPNetwork(cidr)
+      if net.size > 65536:  # greater than /16
+        subnets = net.subnet(16)
+        for subnet_cidr in subnets:
+          cidr_list.append(str(subnet_cidr))
+      else:
+        cidr_list.append(str(cidr))
+
+    cidr_list.sort()
+    ip_set = map(lambda ip: {"Type": "IPV4", "Value": ip}, cidr_list)
+    return json.dumps(list(ip_set))
+
+  def wafv2_cloudflare_ip_list(self):
+    """
+     Args: None
+     Returns:
+       JSON array of Cloudflare IPs in CIDR notation for whitelisting in WAFv2
+    """
+    return json.dumps(list(self.get_cloudflare_cidrs()))
+
+  def wafv2_global_ip_set_arn(self, lookup, default=None):
     """
     Args:
       lookup: the friendly name of a Waf v2 IP set
       default: the optional value to return if lookup failed; returns None if not set
     Returns:
-      the ID of the IP set whose name matches 'lookup' or default/None if no match found
+      the ARN of the WAFv2 Global (Cloudfront) IP set whose name matches 'lookup' or default/None if no match found
+      note that global ARNs are always in us-east-1, which is why the region is hardcoded below
     """
     # list_rules returns at most 100 rules per request
     list_limit = 100
@@ -588,19 +633,20 @@ class EFAwsResolver(object):
     while True:
       for set in ip_sets["IPSets"]:
         if set["Name"] == lookup:
-          return set["Id"]
+          return "arn:aws:wafv2:us-east-1:{{{{ACCOUNT}}}}:global/ipset/{}/{}".format(lookup, set["Id"])
       if "NextMarker" in ip_sets:
         ip_sets = list_ip_sets(Limit=list_limit, Scope="CLOUDFRONT", NextMarker=ip_sets["NextMarker"])
       else:
         return default
 
-  def wafv2_rule_group_id(self, lookup, default=None):
+  def wafv2_global_rule_group_arn(self, lookup, default=None):
     """
     Args:
       lookup: the friendly name of a Waf v2 rule group
       default: the optional value to return if lookup failed; returns None if not set
     Returns:
-      the ID of the WAF v2 rule group whose name matches 'lookup' or default/None if no match found
+      the ARN of the WAFv2 Global (Cloudfront) Rule Group whose name matches 'lookup' or default/None if no match found
+      note that global ARNs are always in us-east-1, which is why the region is hardcoded below
     """
     # list_rules returns at most 100 rules per request
     list_limit = 100
@@ -608,20 +654,21 @@ class EFAwsResolver(object):
     while True:
       for group in rule_groups["RuleGroups"]:
         if group["Name"] == lookup:
-          return group["Id"]
+          return "arn:aws:wafv2:us-east-1:{{{{ACCOUNT}}}}:global/rulegroup/{}/{}".format(lookup, group["Id"])
       if "NextMarker" in rule_groups:
         rule_groups = EFAwsResolver.__CLIENTS["wafv2"].list_rule_groups(
           Limit=list_limit, Scope="CLOUDFRONT", NextMarker=rule_groups["NextMarker"])
       else:
         return default
 
-  def wafv2_web_acl_id(self, lookup, default=None):
+  def wafv2_global_web_acl_arn(self, lookup, default=None):
     """
     Args:
       lookup: the friendly name of a Waf v2 ACL
       default: the optional value to return if lookup failed; returns None if not set
     Returns:
-      the ID of the WAF v2 Web ACL whose name matches rule_name or default/None if no match found
+      the ARN of the WAFv2 Global (Cloudfront) Web ACL whose name matches rule_name or default/None if no match found
+      note that global ARNs are always in us-east-1, which is why the region is hardcoded below
     """
     # list_rules returns at most 100 rules per request
     list_limit = 100
@@ -629,7 +676,7 @@ class EFAwsResolver(object):
     while True:
       for acl in acls["WebACLs"]:
         if acl["Name"] == lookup:
-          return acl["Id"]
+          return "arn:aws:wafv2:us-east-1:{{{{ACCOUNT}}}}:global/webacl/{}/{}".format(lookup, acl["Id"])
       if "NextMarker" in acls:
         acls = EFAwsResolver.__CLIENTS["wafv2"].list_web_acls(
           Limit=list_limit, Scope="CLOUDFRONT", NextMarker=acls["NextMarker"])
@@ -1086,12 +1133,16 @@ class EFAwsResolver(object):
       return self.waf_rule_id(*kv[1:])
     elif kv[0] == "waf:web-acl-id":
       return self.waf_web_acl_id(*kv[1:])
-    elif kv[0] == "wafv2:ip-set-id":
-      return self.wafv2_ip_set_id(*kv[1:])
-    elif kv[0] == "wafv2:rule-group-id":
-      return self.wafv2_rule_group_id(*kv[1:])
-    elif kv[0] == "wafv2:web-acl-id":
-      return self.wafv2_web_acl_id(*kv[1:])
+    elif kv[0] == "waf:cloudflare/ip-list":
+      return self.waf_cloudflare_ip_list()
+    elif kv[0] == "wafv2:global/ip-set-arn":
+      return self.wafv2_global_ip_set_arn(*kv[1:])
+    elif kv[0] == "wafv2:global/rule-group-arn":
+      return self.wafv2_global_rule_group_arn(*kv[1:])
+    elif kv[0] == "wafv2:global/web-acl-arn":
+      return self.wafv2_global_web_acl_arn(*kv[1:])
+    elif kv[0] == "wafv2:cloudflare/ip-list":
+      return self.wafv2_cloudflare_ip_list()
     else:
       return None
       # raise("No lookup function for: "+kv[0])
